import { __awaiter, __rest } from "tslib";
/* eslint-disable react/prop-types */
import * as React from 'react';
import { AppRender } from '../handlers/RenderWrapper';
import { OwnerComponentContext } from '../utils/hooks';
import { ScopedStyle } from './index';
import { ECodeType, EComponentType, processListeners, } from '@cloudbase/cals';
import { createCompApi, app } from '../app/global-api';
import { generateLowcodeModules, generateStyleModuleMap } from '../utils/adapter/lowcode';
import { generateDatasetQuery, getComponentSchema } from '../utils/adapter/generate';
import { runtime } from '../utils/adapter';
import { autorun, observable } from 'mobx';
import jsonSchemaDefaults from 'json-schema-defaults';
import { createWidgets, retryDataBinds, watchAndSyncDatasetState2Local } from '../utils';
import { _WEDA_CLOUD_SDK as WEDA_CLOUD_SDK, getWedaAPI } from '@cloudbase/weda-client';
import { generateDynamicDataset } from '../utils/adapter/datasource';
const { createDataset, createStateDataSourceVar, generateParamsParser, setDatasetProfiles } = WEDA_CLOUD_SDK;
function useLoad(defaultValue, callback, options = {}, deps) {
    const { looseError = false } = options;
    const [loaded, setLoaded] = React.useState(false);
    const [error, setError] = React.useState(null);
    const [data, setData] = React.useState(defaultValue);
    React.useEffect(() => {
        function run() {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    const lowcode = yield callback();
                    setData(lowcode);
                    setLoaded(true);
                    setError(null);
                }
                catch (e) {
                    if (!looseError) {
                        setError(e);
                    }
                    else {
                        console.log(e);
                    }
                    setLoaded(true);
                }
            });
        }
        run();
    }, deps);
    return { data, error, loaded, setData, setError, setLoaded };
}
function loadLowcodesDefult(sourceKey, comp, module, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { processCssUnit = 'rem', lowcodeContext } = options;
        const codeModulesMap = {
            $comp: (comp.resources || []).map((module) => {
                if (module.codeType === ECodeType.THEME || module.codeType === ECodeType.STYLE) {
                    return module;
                }
                const { code } = module, restModule = __rest(module, ["code"]);
                return Object.assign(Object.assign({}, restModule), { code: code.replace(/\$comp/g, 'this.$comp') });
            }),
        };
        const styleModulesMap = {
            $comp: {
                resources: codeModulesMap.$comp,
                processCssUnit,
                cssContextWrapper: `.${getCompositedComponentClass({ module, component: comp.name })}`,
            },
        };
        const [styleCodesMap, { state, computedFuncs, lifeCycle, handler }] = yield Promise.all([
            generateStyleModuleMap(styleModulesMap),
            generateLowcodeModules({
                key: sourceKey,
                codeModulesMap,
                runtimeWindow: runtime,
                context: lowcodeContext,
            }).catch((e) => {
                e.message = `[低码模块加载失败] ${e.absoluteReqPath ? `[${e.absoluteReqPath}]` : ''} ${e.message}`;
                throw e;
            }),
        ]);
        const lowcode = {
            state: state.$comp || {},
            computed: computedFuncs.$comp || {},
            lifecycle: lifeCycle.$comp || {},
            handler: handler.$comp || {},
            style: styleCodesMap.$comp || [],
        };
        return lowcode;
    });
}
const COMPOSITECOMPONENT_DATASETPROFILE_MAP = {};
export function generateCompositeComponent({ sourceKey, comp, module, componentsMap, componentActionsMap, componentsInfoMap, options, loadLowcodes = loadLowcodesDefult, }) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!COMPOSITECOMPONENT_DATASETPROFILE_MAP[sourceKey] && comp.dataset) {
            setDatasetProfiles({
                [sourceKey]: generateDynamicDataset({ looseError: true }, comp.dataset),
            });
        }
        const lowcode = yield loadLowcodes(sourceKey, comp, module, options);
        const Component = React.forwardRef((props, ref) => {
            // const { data, children, $node } = useComponentProps(props as any, true);
            const schema = { items: comp.items, type: EComponentType.COMPLEX, extra: {} };
            return (React.createElement(React.Fragment, null,
                React.createElement(ScopedStyle, { className: `${getCompositedComponentClass({ module, component: comp.name })}-style-tag`, schema: schema, options: options }),
                React.createElement("style", { id: "wd-composite-style-tag", type: "text/css", dangerouslySetInnerHTML: {
                        __html: `${(lowcode.style || []).map((item) => item.code).join('\n')}`,
                    } }),
                React.createElement(CompositeComponent, { ref: ref, sourceKey: sourceKey, comp: comp, props: props, generateLowcodeModule: () => lowcode, generateComponentsMap: () => componentsMap, generateActionsMap: () => componentActionsMap, generateContext: (ctx, comp) => {
                        const wrapperClass = getCompositedComponentClass({ module, component: comp.name });
                        const { widgets, componentSchema, dataBinds } = getComponentSchema(ctx, schema, componentsInfoMap, wrapperClass, {
                            isComposite: true,
                            looseError: options.looseError,
                            processCssUnit: options.processCssUnit,
                            disablePageComponentInvoke: options.disablePageComponentInvoke,
                        });
                        return {
                            widgetContext: widgets,
                            componentSchema,
                            dataBinds,
                        };
                    }, options: options })));
        });
        Component.displayName = comp.name;
        return Component;
    });
}
export function getCompositedComponentClass({ module, component }) {
    return `wa-comp-${module}-${component}`;
}
function useObserverDataRef(data, defaultData) {
    const ref = React.useRef(observable(Object.assign(Object.assign({}, defaultData))));
    const [_, setForceUpate] = React.useReducer((data) => {
        return data + 1;
    }, 0);
    React.useEffect(() => {
        const dispose = autorun(() => {
            let update = false;
            for (const prop in data) {
                // 更新 propsData
                if (typeof data[prop] !== 'function') {
                    ref.current[prop] = data[prop];
                    update = true;
                }
            }
            if (update) {
                setForceUpate();
            }
        });
        return () => {
            dispose();
        };
    }, [data]);
    return { ref, key: _ };
}
const CompositeComponent = React.forwardRef(({ sourceKey, comp: _comp, props, generateComponentsMap, generateLowcodeModule, generateActionsMap, generateContext, options, }, ref) => {
    const { events, $node, data, className, style } = props;
    const { comp, eventFlows = [] } = React.useMemo(() => {
        const { listeners, eventFlows } = processListeners(_comp.listeners || []);
        return { comp: Object.assign(Object.assign({}, _comp), { listeners }), eventFlows };
    }, [_comp]);
    const codeContextRef = React.useRef(null);
    const { ref: dataRef } = useObserverDataRef(data, jsonSchemaDefaults(comp.data));
    const widgetsContextRef = React.useRef({});
    const globalCtxRef = React.useRef({});
    const $comp = codeContextRef.current;
    const [componentCodeContextInited, setcomponentCodeContextInited] = React.useState(false);
    const lowcodeRef = React.useRef(generateLowcodeModule());
    const lowcode = lowcodeRef.current;
    React.useEffect(() => {
        var _a;
        const $comp = createCompApi({
            id: comp.id,
            ref,
            state: observable(lowcode.state),
            computed: lowcode.computed,
            handler: lowcode.handler,
            props,
            propsData: dataRef.current,
            events,
            node: Object.assign($node || {}, {
                getConfig: () => comp.compConfig,
                getOwnerWidget: () => $node.ownerWidget,
                _debug: () => {
                    console.warn('仅用于调试，行为随时发生变化，不要用于生产环境。');
                    return $comp;
                },
            }),
            eventFlows,
        });
        const context = {
            $w: $comp.__internal__.$w,
            $app: app,
            app,
            $page: new Proxy({}, {
                get(target, p) {
                    var _a, _b;
                    return (_b = (_a = getWedaAPI()) === null || _a === void 0 ? void 0 : _a.app.utils.getCurrentPage()) === null || _b === void 0 ? void 0 : _b[p];
                },
            }),
        };
        const dataset = createDataset(sourceKey, undefined, { appId: app === null || app === void 0 ? void 0 : app.id });
        createStateDataSourceVar(sourceKey, generateParamsParser(context));
        dataset.query = generateDatasetQuery((_a = comp.dataset) === null || _a === void 0 ? void 0 : _a.query, context, {
            isComposite: true,
            looseError: true,
            useScopeWidgetData: true,
        });
        $comp.state.dataset = dataset;
        $comp.dataset = dataset;
        let stateSyncDisposes = watchAndSyncDatasetState2Local(app.id, sourceKey, $comp.dataset.state);
        if (options === null || options === void 0 ? void 0 : options.processCssUnit) {
            $comp.__internal__.defaultCssUnit = options === null || options === void 0 ? void 0 : options.processCssUnit;
        }
        codeContextRef.current = $comp;
        setcomponentCodeContextInited(true);
        return () => {
            var _a, _b, _c, _d;
            (_c = (_b = (_a = lowcode.lifecycle.onAttached) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, codeContextRef.current)) === null || _c === void 0 ? void 0 : _c();
            Object.values(((_d = $comp === null || $comp === void 0 ? void 0 : $comp.dataset) === null || _d === void 0 ? void 0 : _d.query) || {}).forEach((query) => {
                var _a;
                (_a = query === null || query === void 0 ? void 0 : query.destroy) === null || _a === void 0 ? void 0 : _a.call(query);
            });
            stateSyncDisposes.forEach((dispose) => {
                dispose === null || dispose === void 0 ? void 0 : dispose();
            });
        };
    }, []);
    const { componentSchema, widgetsContext } = React.useMemo(() => {
        if (componentCodeContextInited) {
            globalCtxRef.current = {
                $comp,
                $app: app,
                actionsMap: (generateActionsMap === null || generateActionsMap === void 0 ? void 0 : generateActionsMap()) || {},
            };
            const globalCtx = globalCtxRef.current;
            let widgetsContext;
            let databinds;
            let componentSchema;
            if (generateContext) {
                let context = generateContext(globalCtx, comp);
                widgetsContext = context.widgetContext;
                databinds = context.dataBinds;
                componentSchema = context.componentSchema;
            }
            // widgets 内的 dataBinds 可能需要关联 widgets，需要重新执行 dataBinds
            $comp._dataBinds = databinds;
            return { componentSchema, widgetsContext };
        }
        return {};
    }, [componentCodeContextInited, comp]);
    widgetsContextRef.current = React.useMemo(() => {
        if (componentCodeContextInited) {
            const globalCtx = globalCtxRef.current;
            const { widgets, nodeMap } = createWidgets({ apiContext: $comp, globalCtx }, widgetsContext, $comp._dataBinds || {}, {}, true);
            Object.entries(widgets).forEach(([key, value]) => {
                $comp.widgets[key] = value;
                const list = !Array.isArray(value) ? [value] : value;
                list.forEach((item) => {
                    item.ownerWidget = $comp.node;
                    item.getOwnerWidget = () => $comp.node;
                });
            });
            setTimeout(retryDataBinds);
            return { widgetProps: widgetsContext, nodeMap, widgets };
        }
        else {
            return widgetsContextRef.current;
        }
    }, [widgetsContext]);
    return (componentCodeContextInited && (React.createElement(CompositeComponentContent, { className: className, style: style, comp: comp, lowcode: lowcode, componentSchema: componentSchema, componentCodeContext: codeContextRef.current, virtualFields: generateComponentsMap() })));
});
CompositeComponent.displayName = 'CompositeComponent';
function CompositeComponentContent({ comp, lowcode, componentSchema, componentCodeContext, virtualFields, className = '', style = {}, }) {
    const { attached } = React.useContext(OwnerComponentContext);
    const [mount, setMounted] = React.useState(false);
    React.useEffect(() => {
        var _a;
        // eslint-disable-next-line no-restricted-syntax
        for (const queryId in ((_a = comp === null || comp === void 0 ? void 0 : comp.dataset) === null || _a === void 0 ? void 0 : _a['query']) || {}) {
            if (comp.dataset['query'][queryId].trigger === 'auto') {
                /**
                 * query 初始化不阻塞生命周期
                 */
                Promise.resolve()
                    .then(() => __awaiter(this, void 0, void 0, function* () {
                    var _b, _c, _d, _e, _f, _g, _h;
                    yield ((_d = (_c = (_b = componentCodeContext === null || componentCodeContext === void 0 ? void 0 : componentCodeContext.dataset) === null || _b === void 0 ? void 0 : _b.query) === null || _c === void 0 ? void 0 : _c[queryId]) === null || _d === void 0 ? void 0 : _d.initPromise);
                    return (_h = (_g = (_f = (_e = componentCodeContext === null || componentCodeContext === void 0 ? void 0 : componentCodeContext.dataset) === null || _e === void 0 ? void 0 : _e.query) === null || _f === void 0 ? void 0 : _f[queryId]) === null || _g === void 0 ? void 0 : _g.trigger) === null || _h === void 0 ? void 0 : _h.call(_g);
                }))
                    .catch((e) => {
                    console.error(`query ${queryId} 初始化失败：`, e);
                });
            }
        }
        setMounted(true);
        return () => {
            var _a, _b, _c;
            if (componentCodeContext.widgets._disposers) {
                componentCodeContext.widgets._disposers.forEach((disposer) => disposer());
                componentCodeContext.widgets._disposers = null;
            }
            (_c = (_b = (_a = lowcode === null || lowcode === void 0 ? void 0 : lowcode.lifecycle.onDetached) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, componentCodeContext)) === null || _c === void 0 ? void 0 : _c();
            componentCodeContext._instanceRef.current = undefined;
            setMounted(false);
        };
    }, []);
    React.useEffect(() => {
        var _a, _b, _c;
        if (attached) {
            (_c = (_b = (_a = lowcode.lifecycle.onReady) === null || _a === void 0 ? void 0 : _a.bind) === null || _b === void 0 ? void 0 : _b.call(_a, componentCodeContext)) === null || _c === void 0 ? void 0 : _c();
        }
    }, [attached]);
    const ownerComponentContext = React.useMemo(() => {
        return { attached: mount };
    }, [mount]);
    return (React.createElement(OwnerComponentContext.Provider, { value: ownerComponentContext },
        React.createElement(AppRender, { className: className, style: style, virtualFields: virtualFields, componentSchema: componentSchema, codeContext: componentCodeContext, isComposite: true })));
}
