/* eslint-disable no-new-func */
import { isEmptyObj, getScopeSlotMeta, isSlot } from '../common';
import { deserializeAttributes, deserializeValue, PropBindType, ActionType, deserializeDynamicMap, EComponentType, processCommonStyle2CSSProperties, processCSSPropertiesUnit, OFFICIAL_LIB_KEY, generateHandlerParamsDynamicMap, } from '@cloudbase/cals';
import { get as lodashGet } from 'lodash-es';
import { runtime } from './index';
import { PAGE_ROOT_SYMBOL, REPEATER } from '../constants';
import { CALC_RPX_ROOT_VAL_NAME } from '../index';
import { Query } from '../query';
import { EventFlow } from '../flow';
import { componentMetaFilter, isRepeaterWidget } from './dependencies';
export function getComponentsInfo(dependencies) {
    const outputObj = {};
    dependencies.forEach(({ name: materialName, components, isComposite }) => {
        components.forEach(({ name, meta, emitEvents, events = undefined, isContainer, compConfig, dataForm, properties, }) => {
            outputObj[`${materialName}:${name}`] = Object.assign(Object.assign({ isComposite }, componentMetaFilter(meta)), { events,
                emitEvents,
                isContainer,
                compConfig,
                dataForm,
                properties });
        });
    });
    return outputObj;
}
function getInputProps(componentsMetaMap) {
    const outputObj = {};
    // eslint-disable-next-line no-restricted-syntax
    for (let key in componentsMetaMap) {
        let component = componentsMetaMap[key];
        const sourceKey = key;
        const inputProps = (component === null || component === void 0 ? void 0 : component.inputProps) || (component === null || component === void 0 ? void 0 : component.syncProps);
        if (inputProps) {
            outputObj[sourceKey] = inputProps;
        }
        let compItem = component;
        Object.keys(compItem.dataForm || {}).forEach((key) => {
            var _a, _b;
            const inputProps = ((_a = compItem.dataForm[key]) === null || _a === void 0 ? void 0 : _a.inputProp) || ((_b = compItem.dataForm[key]) === null || _b === void 0 ? void 0 : _b.syncProp);
            if (inputProps) {
                outputObj[sourceKey] = Object.assign({ [key]: inputProps }, (outputObj[sourceKey] || {}));
            }
        });
    }
    return outputObj;
}
/**
 * utils end
 */
export function splitDynamicMap(data) {
    const staticData = {};
    const dataBinds = [];
    for (let key in data) {
        let meta = data[key];
        if (!meta.type || meta.type === PropBindType.static) {
            staticData[key] = meta.value;
        }
        else if (meta.type === PropBindType.slot) {
            continue;
        }
        else {
            dataBinds.push({
                type: meta.type,
                bindDataPath: meta.value,
                propertyPath: key,
            });
        }
    }
    return { staticData, dataBinds };
}
function walkThroughCals(cals, process, options = {}) {
    var _a, _b;
    let { items = [] } = cals;
    cals.extra = cals.extra || {};
    if (((_a = cals === null || cals === void 0 ? void 0 : cals.directives) === null || _a === void 0 ? void 0 : _a[':if']) === false) {
        cals.items = [];
    }
    const filteredItems = [];
    for (let item of items) {
        if (((_b = item === null || item === void 0 ? void 0 : item.directives) === null || _b === void 0 ? void 0 : _b[':if']) === false) {
            continue;
        }
        filteredItems.push(item);
        item.extra = item.extra || {};
        item.extra.__parent = cals;
    }
    cals.items = filteredItems;
    // 先根遍历 因为父上的状态可能需要带下去
    process(cals, options);
    for (let item of cals.items) {
        walkThroughCals(item, process, options);
    }
    return cals;
}
function parseDirectiveIfAndDispaly(directiveValue) {
    let value;
    switch (directiveValue) {
        case true:
        case false: {
            value = directiveValue;
            break;
        }
        case undefined: {
            value = true;
            break;
        }
    }
    return value;
}
/**
 * widgets 不包括page 的 widges
 */
export function getComponentSchema(ctx, page, componentsInfoMap = {}, wrapperClass, options) {
    const { processCssUnit = 'rem', looseError = false, isComposite = false, disablePageComponentInvoke = false, } = options || {};
    const componentInputProps = getInputProps(componentsInfoMap) || {};
    const copyJson = JSON.parse(JSON.stringify(page, (key, value) => {
        var _a;
        if ((value === null || value === void 0 ? void 0 : value.type) === 'PAGE' && !(value === null || value === void 0 ? void 0 : value.component)) {
            value.component = 'Page';
        }
        /**
         * 处理默认的 module 补全
         */
        if ((value === null || value === void 0 ? void 0 : value.component) && !(value === null || value === void 0 ? void 0 : value.module) && (value === null || value === void 0 ? void 0 : value.type) !== 'PAGE') {
            value.module = OFFICIAL_LIB_KEY;
        }
        if (value === null || value === void 0 ? void 0 : value.component) {
            const component = value;
            const sourceKey = `${component.module}:${component.component}`;
            if (isRepeaterWidget({
                widgetType: sourceKey,
                getConfig: () => {
                    var _a;
                    return ((_a = componentsInfoMap[sourceKey]) === null || _a === void 0 ? void 0 : _a.compConfig) || {};
                },
            }, REPEATER.REPEATER_NAME)) {
                // 给 Repeater 组件加一层虚拟项组件，在虚拟项组件上挂 for 循环
                const { items = [] } = component;
                const firstChildComponent = items[0];
                const sourceKey = `${firstChildComponent === null || firstChildComponent === void 0 ? void 0 : firstChildComponent.module}:${firstChildComponent === null || firstChildComponent === void 0 ? void 0 : firstChildComponent.component}`;
                if (isRepeaterWidget({
                    widgetType: sourceKey,
                    getConfig: () => {
                        var _a;
                        return ((_a = componentsInfoMap[sourceKey]) === null || _a === void 0 ? void 0 : _a.compConfig) || {};
                    },
                }, REPEATER.REPEATER_ITEM_NAME)) {
                    return value;
                }
                component.items = [
                    {
                        component: REPEATER.REPEATER_ITEM_NAME,
                        module: component.module,
                        directives: {
                            ':if': true,
                            ':for': component.attributes[':data'],
                            key: component.attributes.key || '',
                            ':display': (_a = component.directives) === null || _a === void 0 ? void 0 : _a[':display'],
                        },
                        items: items.map((item) => {
                            var _a;
                            let currentDisplay = (_a = component.directives) === null || _a === void 0 ? void 0 : _a[':display'];
                            if (currentDisplay !== undefined && currentDisplay !== 'true') {
                                if (!item.directives) {
                                    item.directives = {};
                                }
                                const current = item.directives[':display'] || 'true';
                                item.directives[':display'] = `(\n${currentDisplay}\n) && (\n${current}\n)`;
                            }
                            return item;
                        }),
                        id: `$weda_internal_${component.id}`, // uniqueId() 解决在编辑器中切换组件导致 children 不断增加的问题
                    },
                ];
            }
        }
        return value;
    }));
    const compWidgets = {};
    const compDataBinds = {};
    const componentSchemaJson = walkThroughCals(copyJson, (component, options = {}) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v;
        const parentComponent = component.extra.__parent;
        if (isSlot(component)) {
            if (parentComponent) {
                const slotId = `${parentComponent.id}.${component.id}`;
                const parentComponentSourceKey = `${parentComponent.module}:${parentComponent.component}`;
                const sourceKey = `${parentComponentSourceKey}.${component.id}`;
                const { isScopeSlot, config: scopeSlotConfig, useScopeWidgetData = false, } = getScopeSlotMeta(parentComponent, component.id, {
                    componentsInfoMap,
                });
                if (isScopeSlot) {
                    compWidgets[slotId] = {
                        _visible: true,
                        _display: true,
                        style: {},
                        classList: [],
                        widgetType: sourceKey,
                        _parentId: parentComponent.id,
                        getConfig: () => ({}),
                        _waForKey: undefined,
                    };
                    if (!useScopeWidgetData) {
                        options.useScopeWidgetData = false;
                    }
                    if ((_a = scopeSlotConfig === null || scopeSlotConfig === void 0 ? void 0 : scopeSlotConfig['x-slot-directives']) === null || _a === void 0 ? void 0 : _a.for) {
                        compDataBinds[slotId] = generateDataBinds(ctx, [
                            {
                                type: PropBindType.expression,
                                bindDataPath: `$w.${parentComponent.id}.${(_b = scopeSlotConfig === null || scopeSlotConfig === void 0 ? void 0 : scopeSlotConfig['x-slot-directives']) === null || _b === void 0 ? void 0 : _b.for}`,
                                propertyPath: '_waFor',
                            },
                        ], {
                            isComposite,
                            looseError,
                            key: `组件 ${slotId}`,
                            useScopeWidgetData: true,
                        });
                    }
                }
            }
            return;
        }
        const { component: componentName, module, attributes = {} } = component;
        const { commonStyle = {} } = component.extra || {};
        const rpxCalcRoot = `var(${CALC_RPX_ROOT_VAL_NAME})`;
        const processedCommonStyle = attributes.style
            ? processCSSPropertiesUnit(attributes.style || {}, processCssUnit, { rpxCalcRoot })
            : processCommonStyle2CSSProperties(commonStyle, {
                defaultUnit: processCssUnit,
                rpxCalcRoot,
            });
        if (processedCommonStyle) {
            component.extra.commonStyle = processedCommonStyle;
        }
        const sourceKey = `${module}:${componentName}`;
        // 如果是复合组件的根节点，则补充 wrapperClass
        if (isComposite) {
            if (((_c = component === null || component === void 0 ? void 0 : component.extra) === null || _c === void 0 ? void 0 : _c.__parent) && !((_f = (_e = (_d = component === null || component === void 0 ? void 0 : component.extra) === null || _d === void 0 ? void 0 : _d.__parent) === null || _e === void 0 ? void 0 : _e.extra) === null || _f === void 0 ? void 0 : _f.__parent)) {
                if (!component.attributes) {
                    component.attributes = {};
                }
                component.attributes.class = Array.from(new Set([component.attributes.class, wrapperClass === null || wrapperClass === void 0 ? void 0 : wrapperClass.split(/\s/)]))
                    .filter((item) => !!item)
                    .join(' ');
            }
        }
        const { class: className = '' } = attributes;
        const deserializeCtx = isComposite ? { page: page } : {};
        let dynamicStyle = component[':style']
            ? deserializeValue(deserializeCtx, ':style', component[':style']).value
            : undefined;
        let dynamicClassList = component[':class']
            ? deserializeValue(deserializeCtx, ':class', component[':class']).value
            : undefined;
        let dynamicIf = ((_g = component.directives) === null || _g === void 0 ? void 0 : _g[':if'])
            ? deserializeValue(deserializeCtx, ':if', (_h = component.directives) === null || _h === void 0 ? void 0 : _h[':if']).value
            : undefined;
        let dynamicDisplay = ((_j = component.directives) === null || _j === void 0 ? void 0 : _j[':display'])
            ? deserializeValue(deserializeCtx, ':display', (_k = component.directives) === null || _k === void 0 ? void 0 : _k[':display']).value
            : undefined;
        let dynamicFor = ((_l = component.directives) === null || _l === void 0 ? void 0 : _l[':for'])
            ? deserializeValue(deserializeCtx, ':for', (_m = component.directives) === null || _m === void 0 ? void 0 : _m[':for']).value
            : undefined;
        if (dynamicIf != undefined &&
            (dynamicIf.value == undefined || dynamicIf.value === true || dynamicIf.value === false)) {
            dynamicIf = undefined;
        }
        if (dynamicDisplay != undefined &&
            (dynamicDisplay.value == undefined || dynamicDisplay.value === true || dynamicDisplay.value === false)) {
            dynamicDisplay = undefined;
        }
        if (dynamicFor) {
            let { type, value } = dynamicFor;
            if (type === PropBindType.expression) {
                dynamicFor.value = value.replace(/^(.*?in\s)/, '');
            }
        }
        const data = deserializeAttributes(deserializeCtx, attributes);
        const { dataBinds, staticData } = splitDynamicMap(data);
        const componentInfo = componentsInfoMap[sourceKey];
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig) {
            component.extra.compConfig = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig;
        }
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events) {
            component.extra.emitEvents = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events.map((item) => item.name);
        }
        else if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.emitEvents) {
            component.extra.emitEvents = componentInfo.emitEvents.map((item) => item.eventName);
        }
        // 生成 widgets/dataBinds
        if (!isSlot(component) && component.id) {
            let _visible = parseDirectiveIfAndDispaly((_o = component.directives) === null || _o === void 0 ? void 0 : _o[':if']);
            let _display = parseDirectiveIfAndDispaly((_p = component === null || component === void 0 ? void 0 : component.directives) === null || _p === void 0 ? void 0 : _p[':display']);
            if (component.type !== 'PAGE') {
                /**
                 * TODO:
                 * 原版中 _parentId 赋值
                 *  _parentId: parentComponent
                 *  ? isSlot(parentComponent)
                 *    ? parentComponent.extra?.__parent?.id
                 *   : parentComponent.id
                 * : null,
                 */
                let _parentId = null;
                if (parentComponent) {
                    _parentId = parentComponent.id;
                    if (parentComponent.type === EComponentType.PAGE || parentComponent.type === EComponentType.COMPLEX) {
                        _parentId = (_r = (_q = parentComponent.extra) === null || _q === void 0 ? void 0 : _q.__parent) === null || _r === void 0 ? void 0 : _r.id;
                    }
                    else if (isSlot(parentComponent)) {
                        _parentId = (_t = (_s = parentComponent.extra) === null || _s === void 0 ? void 0 : _s.__parent) === null || _t === void 0 ? void 0 : _t.id;
                        const { isScopeSlot } = getScopeSlotMeta(parentComponent.extra.__parent, parentComponent.id, {
                            componentsInfoMap,
                        });
                        if (isScopeSlot) {
                            _parentId = `${(_u = parentComponent.extra.__parent) === null || _u === void 0 ? void 0 : _u.id}.${parentComponent.id}`;
                        }
                    }
                }
                compWidgets[component.id] = Object.assign(Object.assign({}, Object.entries(staticData).reduce((data, [propsKey, propsValue]) => {
                    let value = propsValue;
                    if (value && Array.isArray(value)) {
                        value = value.map((_, index) => {
                            var _a, _b;
                            const block = (_a = attributes[propsKey]) === null || _a === void 0 ? void 0 : _a[index]; // 因为 value 里的是序列化过的，尤其是选区的是时候
                            const blockInstances = (_b = block === null || block === void 0 ? void 0 : block.selectableBlock) === null || _b === void 0 ? void 0 : _b.listeners;
                            if (blockInstances) {
                                block.selectableBlock.listenerInstances = generateListnerInstances(ctx, blockInstances || [], {
                                    isComposite,
                                    looseError,
                                    useScopeWidgetData: options.useScopeWidgetData === undefined ? true : options.useScopeWidgetData,
                                    disablePageComponentInvoke,
                                });
                                return block;
                            }
                            else {
                                // 普通的非选区的，返回staticData 里的值
                                return _;
                            }
                        });
                    }
                    data[propsKey] = value;
                    return data;
                }, {})), { 
                    /**
                     * 此处新增key，需要同步定义 widgets.ts/widgetKeys 常量
                     */
                    _visible,
                    _display, style: Object.assign({}, processedCommonStyle), classList: className.split(/\s/).filter((item) => !!item), widgetType: sourceKey, _parentId, getConfig: () => {
                        var _a;
                        return (_a = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.compConfig) !== null && _a !== void 0 ? _a : {};
                    }, _waForKey: (_v = component.directives) === null || _v === void 0 ? void 0 : _v.key });
            }
            const processedDataBinds = [...dataBinds];
            const dynamicMetaList = [
                { property: '_visible', meta: dynamicIf },
                { property: '_display', meta: dynamicDisplay },
                { property: '_waFor', meta: dynamicFor },
                { property: 'style', meta: dynamicStyle },
                { property: 'classList', meta: dynamicClassList },
            ];
            for (let item of dynamicMetaList) {
                const { property, meta } = item;
                if (meta === null || meta === void 0 ? void 0 : meta.value) {
                    processedDataBinds.push({
                        type: meta.type,
                        bindDataPath: meta.value,
                        propertyPath: property,
                    });
                }
            }
            if (processedDataBinds.length > 0) {
                let databinds = generateDataBinds(ctx, processedDataBinds, {
                    isComposite,
                    looseError,
                    key: `组件 ${component.id}`,
                    useScopeWidgetData: options.useScopeWidgetData === undefined ? true : options.useScopeWidgetData,
                });
                compDataBinds[component.id] = databinds;
                if (component.type === 'PAGE') {
                    compDataBinds[PAGE_ROOT_SYMBOL] = databinds;
                }
            }
        }
        component.listenerInstances = generateListnerInstances(ctx, component.listeners || [], {
            isComposite,
            looseError,
            key: `组件事件 ${component.id}`,
            useScopeWidgetData: options.useScopeWidgetData === undefined ? true : options.useScopeWidgetData,
            disablePageComponentInvoke: options.disablePageComponentInvoke,
        });
        delete component.listeners;
        if (!isEmptyObj(attributes)) {
            // 组件双向绑定
            const syncProps = componentInputProps[sourceKey];
            if (syncProps) {
                Object.keys(syncProps).forEach((syncPropKey) => {
                    const syncProp = syncProps[syncPropKey];
                    let syncPropArr = syncProp !== null && syncProp !== void 0 ? syncProp : [];
                    // 统一转成数组处理
                    if (!Array.isArray(syncProp)) {
                        syncPropArr = [syncProp];
                    }
                    else {
                        syncPropArr = syncPropArr;
                    }
                    syncPropArr.forEach(({ changeEvent, valueFromEvent }) => {
                        // 双向绑定需要优先第一个执行，否则会出现输入框的拼音被打断的问题
                        function withContext(context) {
                            let instance = new runtime.Function('context', `return function({ event, forItems }) {
                  const $for = forItems;
                  const $index = $for.forIndexes ? $for.forIndexes[$for.forIndexes.length - 1] : undefined;
                  const wid = ${isComposite ? 'this.widgets' : 'context.$page.widgets'}.${component.id};
                  const widgetData = $for.forIndexes && $for.forIndexes.length > 0 ? context.get(wid, forItems.forIndexes) : wid;
                  widgetData.${syncPropKey} = ${valueFromEvent}
                }`)(context);
                            return ctx.this ? instance.bind(this) : instance;
                        }
                        component.listenerInstances.unshift({
                            trigger: changeEvent,
                            instanceFunction: withContext(Object.assign(Object.assign({}, ctx), { get: lodashGet })),
                        });
                    });
                });
            }
        }
    });
    return {
        widgets: compWidgets,
        dataBinds: compDataBinds,
        componentSchema: componentSchemaJson,
    };
}
/**
 * convert data binds to functions for performance & simplicity
 * generateListener 中 GeneralFunc 参数保持一致
 */
export function generateDataBinds(ctx, dataBinds, options) {
    const dataBindFuncs = {};
    const { useScopeWidgetData = true } = options || {};
    dataBinds.forEach((bind) => {
        var _a, _b;
        if (!bind.bindDataPath) {
            return console.warn('bindDataPath error', bind.propertyPath);
        }
        let databindFunction = () => { };
        let funcCode = `function (){}`;
        const alias = `
const $for = forItems;
const $index = $for.forIndexes ? $for.forIndexes[$for.forIndexes.length - 1] : undefined;
`;
        try {
            if (bind.type === PropBindType.forItem) {
                funcCode = `function(forItems){ return forItems.${bind.bindDataPath} } `;
            }
            else if (bind.type === 'scope') {
                funcCode = `function(_, __, ___, ____, $scope, _____){ ${useScopeWidgetData
                    ? 'return $scope.${bind.bindDataPath};'
                    : `return ({__type: "scopedValue", getValue: function($scope){ return $scope.${bind.bindDataPath}}});`} }`;
            }
            else if (bind.type === PropBindType.expression) {
                const code = bind.bindDataPath.replace(/;$/, '');
                const content = `${alias} return (\n${code || 'undefined'}\n)`;
                if ((!useScopeWidgetData && /\$scope\./.test(code)) || code === '$scope') {
                    funcCode = `(forItems, event, $context, $w, $scope, params) => ({__type: "scopedValue", getValue: ($scope) => { ${content} }})`;
                }
                else {
                    funcCode = `(forItems, event, $context, $w, $scope, params) => {${content}}`;
                }
            }
            else if (bind.type === PropBindType.prop) {
                let { bindDataPath } = bind;
                const isNegated = bindDataPath.startsWith('!');
                if (isNegated)
                    bindDataPath = bindDataPath.replace(/^!/, '');
                // 复合组件在预览时其实就是page，所以沿用page的变量即可
                funcCode = `function() { return ${isNegated ? '!' : ''}${(options === null || options === void 0 ? void 0 : options.isComposite) ? '$comp' : '$page'}.props.data.${bindDataPath} }`;
            }
            else {
                const isGlobal = bind.bindDataPath.startsWith('global.');
                let returnValue = '';
                const PREFIX_MAP = {
                    [PropBindType.state]: 'state',
                    [PropBindType.computed]: 'computed',
                    // [PropBindType.dataVar]: 'dataVar',
                    [PropBindType.stateData]: 'dataset.state',
                    [PropBindType.paramData]: 'dataset.params',
                };
                switch (bind.type) {
                    case PropBindType.state:
                    case PropBindType.computed:
                    // case PropBindType.dataVar:
                    case PropBindType.stateData:
                    case PropBindType.paramData: {
                        if (isGlobal) {
                            returnValue = bind.bindDataPath.replace(/^global([.[])/, (_, $1) => `$app.${PREFIX_MAP[bind.type]}${$1}`);
                        }
                        else {
                            if (options === null || options === void 0 ? void 0 : options.isComposite) {
                                returnValue = bind.bindDataPath
                                    .replace(/^comp-\w+([.[])/, (_, $1) => `$comp.${PREFIX_MAP[bind.type]}${$1}`)
                                    .replace(/^\$\w+_\d+([.[])/, (_, $1) => `$comp.${PREFIX_MAP[bind.type]}${$1}`);
                            }
                            else {
                                returnValue = bind.bindDataPath.replace(/^\w+([.[])/, (_, $1) => `$page.${PREFIX_MAP[bind.type]}${$1}`);
                            }
                        }
                        funcCode = `function(){ return ${returnValue} }`;
                        break;
                    }
                }
            }
            const imports = Object.keys(((_a = ctx === null || ctx === void 0 ? void 0 : ctx.$app) === null || _a === void 0 ? void 0 : _a.utils) || {}).map((key) => {
                return `const ${key} = $app.utils["${key}"]`;
            });
            if (imports.length) {
                imports.push('');
            }
            try {
                databindFunction = new runtime.Function('context', `const { $page, $app, $comp } = context || {};
          const app = $app;
          ${imports.join(';')}
          return ${funcCode}`)(ctx);
            }
            catch (e) {
                let errorType = e instanceof SyntaxError ? '语法错误' : '错误';
                e.message = `${options.key ? `${options.key}.` : ''}${bind.propertyPath} 绑定${errorType}: ${(_b = e.message) === null || _b === void 0 ? void 0 : _b.replace(/^Unexpected token/, '非法字符')}`;
                throw e;
            }
            if (ctx.this) {
                databindFunction = databindFunction.bind(ctx.this);
            }
        }
        catch (e) {
            if (options === null || options === void 0 ? void 0 : options.looseError) {
                console.error('generate dynamic data error:', e);
                databindFunction = () => {
                    throw e;
                };
            }
            else {
                throw e;
            }
        }
        dataBindFuncs[bind.propertyPath] = databindFunction;
    });
    return dataBindFuncs;
}
/**
 * dataset 特殊的绑定格式
 * 要求：入参分别为 app ， page
 * 绑定只有变量，参数，表达式的绑定
 */
export function generateDataBindForDataset(bind) {
    let expr = '';
    const { type, bindDataPath } = bind;
    if (type === PropBindType.static || !type) {
        expr = typeof bindDataPath === 'string' ? `'${bindDataPath}'` : JSON.stringify(bindDataPath);
    }
    else if (type === PropBindType.expression) {
        expr = bindDataPath;
    }
    else {
        const isGlobal = bindDataPath.startsWith('global.');
        const isCopositive = bindDataPath.startsWith('comp-') || bindDataPath.startsWith('$comp_');
        const PREFIX_MAP = {
            [PropBindType.state]: 'state',
            [PropBindType.computed]: 'computed',
            [PropBindType.dataVar]: 'dataVar',
            [PropBindType.stateData]: 'dataset.state',
            [PropBindType.paramData]: 'dataset.params',
        };
        switch (type) {
            case PropBindType.state:
            case PropBindType.computed:
            case PropBindType.dataVar:
            case PropBindType.stateData:
            case PropBindType.paramData: {
                if (isCopositive) {
                    expr = bindDataPath
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
                else {
                    expr = bindDataPath
                        .replace(/^\w+./, `${isGlobal ? 'app' : '$page'}.${PREFIX_MAP[type]}.`)
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
            }
        }
    }
    return new runtime.Function('app, $page, $w', `const $app = app;
    const imports = Object.keys(app && app.utils || {}).map(function (key) {
      return \`const \${key} = app.utils["\${key}"]\`;
    });
    if(imports.length) {
      imports.push('')
    }
    const body =  imports.join(';')+\`return (\n${expr || 'undefined'}\n)\`
   return new Function("", body)()`);
}
/**
 * 修改该函数需要同步修改
 * @cloudbase/lowcode-generator
 * src/generator/core/generateCals.ts/genearteListenerInstances
 */
export function generateListnerInstances(ctx, listenerInstances, options = {
    isComposite: false,
    key: '',
    useScopeWidgetData: true,
    disablePageComponentInvoke: false,
}) {
    const { isComposite = false, disablePageComponentInvoke = false } = options;
    return listenerInstances.map((listener) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const generatedListener = {
            key: listener.id,
            trigger: listener.eventName,
            isCapturePhase: !!listener.isCapturePhase,
            noPropagation: listener.noPropagation,
            instanceFunction: () => { },
            data: {},
            dataBinds: {},
        };
        switch (listener.type) {
            case ActionType.Material: {
                const { name, module } = listener.handler || {};
                generatedListener.instanceFunction = new runtime.Function('context', `const { $page, $app, actionsMap = {} } = context || {};
        const app = $app;
        const sourceKey = "${module}:${name}";
        return function(...args){ return actionsMap[sourceKey] &&  actionsMap[sourceKey].apply(null, args)} `)(ctx);
                break;
            }
            case ActionType.PropEvent: {
                if (isComposite) {
                    generatedListener.instanceFunction = new runtime.Function('context', `return function ({ data, event }) {
              event.detail.data = data
              return context.$comp.props.emit('${listener.handler.name}', event.detail);}`)(ctx);
                }
                else {
                    generatedListener.instanceFunction = new runtime.Function('context', `return function (...args) { return context.$page.props.events.${(_a = listener.handler) === null || _a === void 0 ? void 0 : _a.name}.apply(null, args);}`)(ctx);
                }
                break;
            }
            case ActionType.Inline: {
                // 内联 handler
                const instance = new runtime.Function('{event, data, forItems, $w, args}', `return ${listener.jsCode}`);
                generatedListener.instanceFunction = instance.bind(isComposite ? ctx.$comp : ctx.$page);
                break;
            }
            case ActionType.GeneralFunc: {
                const code = ((_b = listener.handler) === null || _b === void 0 ? void 0 : _b[':code']) || ((_c = listener.handler) === null || _c === void 0 ? void 0 : _c.code);
                // 普通匿名函数
                const instance = new runtime.Function(
                /**
                 * 注入类似 databinds 作用域
                 * $w, params
                 * $context, $scope, forItems 通过 $w 实现不再pick
                 * $item $index 不再 alias
                 * ABS 等方法不再直接支持
                 * code 为兼容之前写法,因为实际上是函数引用，因此使用表达式更合适
                 */
                '{event, $w, params}', `${isComposite ? 'const $comp = this;' : ''} return (\n${((_d = code === null || code === void 0 ? void 0 : code.replace) === null || _d === void 0 ? void 0 : _d.call(code, /;\s*$/, '')) || '()=>{}'}\n)({event})`);
                generatedListener.instanceFunction = instance.bind(isComposite ? ctx.$comp : ctx.$page);
                break;
            }
            case ActionType.Platform: {
                let name = ((_e = listener.handler) === null || _e === void 0 ? void 0 : _e.name) || listener.jsCode;
                generatedListener.sourceKey = `${((_f = listener.handler) === null || _f === void 0 ? void 0 : _f.module) || ''}:${((_g = listener.handler) === null || _g === void 0 ? void 0 : _g.name) || ''}`;
                let functionBody = '';
                switch (name) {
                    case 'callNanoFlow': {
                        functionBody = `const flow = ${isComposite ? 'context.$comp.__internal__.$w[data.id]' : 'context.$page.__internal__.$w[data.id]'}; return flow && flow.trigger(data.data)`;
                        break;
                    }
                    case 'callQuery': {
                        functionBody = `const query = ${isComposite ? 'context.$comp.__internal__.$w[data.id]' : 'context.$page.__internal__.$w[data.id]'}; return query && query[data.method](data.data)`;
                        break;
                    }
                    case 'invoke': {
                        functionBody = disablePageComponentInvoke
                            ? `const func = $w[data.component]?.[data.method];
                if (typeof func !== 'function') {
                  throw new Error(\`调用方法失败：未找到id为\${data.component}下的方法\${data.method}\`);
                }
                return func(data.params)`
                            : `return context.$app.invoke({...data, component: $w[data.component]})`;
                        break;
                    }
                    case 'setCurrentPageParams': {
                        functionBody = `return context.$app.utils.getCurrentPage().setParams(data.params);`;
                        break;
                    }
                    case 'ai.LLM.chat':
                    case 'ai.bot.sendMessage':
                    case 'ai.bot.getRecommendQuestions': {
                        functionBody = ` return context.$app.${name}(data, $w) `;
                        break;
                    }
                    default: {
                        functionBody = ` return context.$app.${name}(...args) `;
                    }
                }
                generatedListener.instanceFunction = new runtime.Function('context', `return function({data, $w, args}){ ${functionBody} } `)(ctx);
                break;
            }
            case ActionType.DataSource: {
                generatedListener.instanceFunction = new runtime.Function('context', `return function({args}) {return context.$app.cloud.callDataSource(...args)}`)(ctx);
                break;
            }
            default: {
                // Lowcode action(handler)
                if (isComposite) {
                    generatedListener.instanceFunction = new runtime.Function('context', `return context.$comp.handler.${((_h = listener.handler) === null || _h === void 0 ? void 0 : _h.name) || listener.jsCode}.bind(context.$comp)`)(ctx);
                }
                else {
                    let name = ((_j = listener.handler) === null || _j === void 0 ? void 0 : _j.name) || listener.jsCode;
                    generatedListener.instanceFunction = new runtime.Function('context', `return context.$page.handler.${name}`)(ctx);
                }
                break;
            }
        }
        const processedParams = deserializeDynamicMap({
            page: isComposite ? undefined : { id: ctx.$page.id, type: EComponentType.PAGE },
        }, generateHandlerParamsDynamicMap(listener.handler));
        const { staticData, dataBinds } = splitDynamicMap(processedParams);
        generatedListener.data = staticData;
        generatedListener.dataBinds = generateDataBinds(ctx, dataBinds, Object.assign(Object.assign({}, options), { key: `${options.key ? `${options.key}.` : ''}listener.${listener.eventName}` }));
        return generatedListener;
    });
}
export function generateDatasetQuery(schema, context, options) {
    const result = {};
    for (const key in schema) {
        result[key] = new Query({ schema: schema[key], context, options });
    }
    return result;
}
export function generateEventFlows(flows = [], context, options) {
    const result = {};
    for (let flow of flows) {
        result[flow.id] = new EventFlow({ schema: flow, context, options });
    }
    return result;
}
