"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchAndSyncDatasetState2Local = exports.createComputed = void 0;
const mobx_1 = require("mobx");
const weda_client_1 = require("@cloudbase/weda-client");
const { getDatasetProfiles, setLocalDatasetState } = weda_client_1._WEDA_CLOUD_SDK;
/* eslint-disable no-restricted-syntax */
function createComputed(funcs, bindContext = null) {
    const obj = {};
    for (const name in funcs) {
        Object.defineProperty(obj, name, {
            get: bindContext ? funcs[name].bind(bindContext) : funcs[name],
        });
    }
    return obj;
}
exports.createComputed = createComputed;
function watchAndSyncDatasetState2Local(appId, datasetProfileKey, state) {
    var _a;
    const disposes = [];
    const stateProfile = ((_a = getDatasetProfiles(datasetProfileKey)) === null || _a === void 0 ? void 0 : _a.state) || {};
    for (const key in stateProfile) {
        const config = stateProfile[key];
        if (config.enableSyncLocal) {
            let init = false;
            let timer = null;
            const dispose = (0, mobx_1.autorun)((r) => {
                const value = state[key];
                if (timer) {
                    clearTimeout(timer);
                }
                timer = setTimeout(() => {
                    if (init) {
                        try {
                            setLocalDatasetState(appId, datasetProfileKey, key, value);
                        }
                        catch (e) {
                            console.error('setLocalDatasetState error:', e);
                        }
                    }
                }, 50);
                init = true;
            });
            disposes.push(dispose);
        }
    }
    return disposes;
}
exports.watchAndSyncDatasetState2Local = watchAndSyncDatasetState2Local;
//# sourceMappingURL=computed.js.map